{*******************************************************}
{                                                       }
{               5x7 Matrix LCD Component                }
{                                                       }
{*******************************************************}
unit uLCD5x7;

interface

uses
  Classes, Messages, SysUtils, Windows, Graphics, Controls, ExtCtrls;

const
  LCD_CHAR_WIDTH   =  5;
  
  LCD_DEFAULT_COLS = 16;
  LCD_DEFAULT_ROWS =  2;
  LCD_DEFAULT_CELL =  3;
  LCD_DEFAULT_GAP  =  1;

  LCD_DEFAULT_FORECOLOR  = $00303020;
  LCD_DEFAULT_BACKCOLOR  = $0070A080;
  LCD_DEFAULT_PANELCOLOR = $0080B090;

type
  {$IFDEF UNICODE}
  TRawString = RawByteString;
  {$ELSE}
  TRawString = AnsiString;
  {$ENDIF}

  TFontImage = array [0..LCD_CHAR_WIDTH-1] of Byte;

  TLCDColor = (lcdcCustom, lcdcStandard, lcdcWhiteOnBlue, lcdcWhiteOnBlack,
    lcdcBlackOnOrange, lcdcBlackOnWhite);

  TLCDCursorType = (lcduUnderline, lcduDoubleUnderLine, lcduFullMark,
    lcduFullSpace, lcduInsert);

  TLCDMatrixType = (lcdm5x7, lcdm5x8);

  TLCDCursorPos = class (TPersistent)
  private
    FX: Byte;
    FY: Byte;
    FMaxX: Byte;
    FMaxY: Byte;
    procedure SetX(const Value: Byte);
    procedure SetY(const Value: Byte);
    procedure SetMaxX(const Value: Byte);
    procedure SetMaxY(const Value: Byte);
  public
    { Constructor / Destructor }
    constructor Create;
    destructor Destroy; override;
  public
    property MaxX: Byte read FMaxX write SetMaxX;
    property MaxY: Byte read FMaxY write SetMaxY;
  published
    property X: Byte read FX write SetX default 0;
    property Y: Byte read FY write SetY default 0;
  end;

  {$IF CompilerVersion >= 23.0}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$IFEND}
  TLCD5x7 = class (TGraphicControl)
  private
    { Fields }
    FBevelSize: Byte;
    FBitmap: TBitmap;
    FBlink: Boolean;
    FBlinkTimer: TTimer;
    FCellGap: Byte;
    FCellSize: Byte;
    FCharBuf: array of array of Byte;
    FCursorImages: array [TLCDCursorType] of TFontImage;
    FFontImages: array [Byte] of TFontImage;
    FHeight: Integer;
    FLCDBackColor: TColor;
    FLCDColor: TLCDColor;
    FLCDColumns: Byte;
    FLCDCursorBlink: Boolean;
    FLCDCursorPos: TLCDCursorPos;
    FLCDCursors: array [TLCDCursorType] of TBitmap;
    FLCDCursorType: TLCDCursorType;
    FLCDFonts: array [Byte] of TBitmap;
    FLCDForeColor: TColor;
    FLCDMatrixHeight: Byte;
    FLCDPanelColor: TColor;
    FLCDRows: Byte;
    FMatrixType: TLCDMatrixType;
    FOnPaint: TNotifyEvent;
    FPadding: TPadding;
    FShowCursor: Boolean;
    FWidth: Integer;
    FLCDMatrixType: TLCDMatrixType;
    { Access Methods }
    function GetBitmap: TBitmap;
    procedure SetCellGap(const Value: Byte);
    procedure SetCellSize(const Value: Byte);
    procedure SetLCDBackColor(const Value: TColor);
    procedure SetLCDColor(const Value: TLCDColor);
    procedure SetLCDColumns(const Value: Byte);
    procedure SetLCDForeColor(const Value: TColor);
    procedure SetLCDPanelColor(const Value: TColor);
    procedure SetLCDRows(const Value: Byte);
    procedure SetPadding(const Value: TPadding);
    { Methods }
    procedure _ClearLine(y: Byte);
    procedure _GenerateFont(ab: array of TBitmap; af: array of TFontImage; aIndex: Byte);
    procedure ChangeBuf;
    procedure ChangeSize;
    procedure ChangeLCDColor;
    procedure DoSizeChange(Sender: TObject);
    procedure SetShowCursor(const Value: Boolean);
    { Message Methods }
    {$IF CompilerVersion >= 21.0}
    procedure CMGestureManagerChanged(var Message: TMessage); message CM_GESTUREMANAGERCHANGED;
    {$IFEND}
    { Event Handler}
    procedure BlinkTimer(Sender: TObject);
    procedure SetCursorBlink(const Value: Boolean);
    procedure SetCursorPos(const Value: TLCDCursorPos);
    procedure SetCursorType(const Value: TLCDCursorType);
    procedure SetCursorImages;
    procedure SetCDMatrixType(const Value: TLCDMatrixType);
  protected
    { Methods }
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    procedure Paint; override;
  public
    { Constructor / Destructor }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    { Methods }
    procedure APrint(address: word; const s: TRawString); overload;
    procedure ClearLine(y: Byte);
    procedure ClearScreen;
    procedure DeleteLine(y: Byte; n: Byte);
    procedure GenerateFont(aIndex: Byte); overload;
    procedure GenerateFont; overload;
    procedure InsertLine(y: Byte; n: Byte);
    procedure LPrint(x, y: Byte; const s: TRawString);
    procedure ScrollUp(n: Byte);
    procedure ScrollDown(n: Byte);
    procedure SetCursorImage(aCursor: TLCDCursorType; Image1, Image2, Image3, Image4, Image5: Byte; Generate: Boolean = True);
    procedure SetFontImage(aIndex: Byte; Image1, Image2, Image3, Image4, Image5: Byte; Generate: Boolean = True);
    { Properties }
    property Bitmap: TBitmap read GetBitmap;
    property Canvas;
  published
    { Properties }
    property CellGap: Byte read FCellGap write SetCellGap default LCD_DEFAULT_GAP;
    property CellSize: Byte read FCellSize write SetCellSize default LCD_DEFAULT_CELL;
    property CursorPos: TLCDCursorPos read FLCDCursorPos write SetCursorPos;
    property CursorType: TLCDCursorType read FLCDCursorType write SetCursorType default lcduFullMark;
    property CursorBlink: Boolean read FLCDCursorBlink write SetCursorBlink default True;
    property LCDBackColor: TColor read FLCDBackColor write SetLCDBackColor default LCD_DEFAULT_BACKCOLOR;
    property LCDColor: TLCDColor read FLCDColor write SetLCDColor default lcdcStandard;
    property LCDColumns: Byte read FLCDColumns write SetLCDColumns default LCD_DEFAULT_COLS;
    property LCDForeColor: TColor read FLCDForeColor write SetLCDForeColor default LCD_DEFAULT_FORECOLOR;
    property LCDMatrixType: TLCDMatrixType read FLCDMatrixType write SetCDMatrixType default lcdm5x7;
    property LCDPanelColor: TColor read FLCDPanelColor write SetLCDPanelColor default LCD_DEFAULT_PANELCOLOR;
    property LCDRows: Byte read FLCDRows write SetLCDRows default LCD_DEFAULT_ROWS;
    property Padding: TPadding read FPadding write SetPadding;
    property ShowCursor: Boolean read FShowCursor write SetShowCursor default False;
    { Properties }
    property Align;
    property Anchors;
    property AutoSize default True;
    property Constraints;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    {$IF CompilerVersion >= 21.0}
    property Touch;
    {$IFEND}
    property Visible;
    { Events }
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    { Events }
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    {$IF CompilerVersion >= 21.0}
    property OnGesture;
    {$IFEND}
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

{ TLCDCursorPos }

constructor TLCDCursorPos.Create;
begin
  inherited;
  FX := 0;
  FY := 0;
end;

destructor TLCDCursorPos.Destroy;
begin
  inherited;
end;

procedure TLCDCursorPos.SetMaxX(const Value: Byte);
begin
  if (FMaxX = Value) then
    Exit;
  FMaxX := Value;
  if FX > FMaxX then
    FX := FMaxX;
end;

procedure TLCDCursorPos.SetMaxY(const Value: Byte);
begin
  if (FMaxY = Value) then
    Exit;
  FMaxY := Value;
  if FY > FMaxY then
    FY := FMaxY;
end;

procedure TLCDCursorPos.SetX(const Value: Byte);
begin
  if (FX = Value) or (Value > MaxX) then
    Exit;
  FX := Value;
end;

procedure TLCDCursorPos.SetY(const Value: Byte);
begin
  if (FY = Value) or (Value > MaxY) then
    Exit;
  FY := Value;
end;

{ TLCD5x7 }

// -------------------------
// Constructor / Destructor
// -------------------------

constructor TLCD5x7.Create(AOwner: TComponent);
var
  i: Integer;
  c: TLCDCursorType;
begin
  Inherited Create(AOwner);

  // Matrix
  FLCdMatrixHeight := 7;
  FLCDMatrixType := lcdm5x7;

  // Padding
  FPadding := TPadding.Create(Self);
  FPadding.OnChange := DoSizeChange;

  // Style
  FLCDColumns := LCD_DEFAULT_COLS;
  FLCDRows    := LCD_DEFAULT_ROWS;
  FCellSize   := LCD_DEFAULT_CELL;
  FCellGap    := LCD_DEFAULT_GAP;
  FBevelSize  := FCellSize + FCellGap;
  ChangeBuf;

  AutoSize := True;
  ChangeSize;

  FLCDColor   := lcdcStandard;
  ChangeLCDColor;

  // Fonts
  for i := Low(FLCDFonts) to High(FLCDFonts) do
    FLCDFonts[i] := TBitmap.Create;

  {$I 'uLCD5x7_sc1602b.inc'}

  for c := Low(FLCDCursors) to High(FLCDCursors) do
    FLCDCursors[c] := TBitmap.Create;


  SetCursorImage(lcduFullMark       , $FF, $FF, $FF, $FF, $FF, False);
  SetCursorImage(lcduFullSpace      , $00, $00, $00, $00, $00, False);
  SetCursorImage(lcduInsert         , $08, $1C, $3E, $7F, $00, False);
  SetCursorImages;

  GenerateFont;

  // Cursor
  FShowCursor          := False;
  FLCDCursorPos        := TLCDCursorPos.Create;
  FLCDCursorPos.MaxX   := FLCDColumns - 1;
  FLCDCursorPos.MaxY   := FLCDRows    - 1;
  FLCDCursorType       := lcduFullMark;
  FLCDCursorBlink      := True;
  FLCDCursorPos.X      := 0;
  FLCDCursorPos.Y      := 0;
  FBlinkTimer          := TTimer.Create(Self);
  FBlinkTimer.Interval := 375;
  FBlinkTimer.OnTimer  := BlinkTimer;

  // Bitmap
  FBitmap := TBitmap.Create;
end;

destructor TLCD5x7.Destroy;
var
  i: Integer;
  c: TLCDCursorType;
begin
  FBitmap.Free;
  FBlinkTimer.Free;
  for i := Low(FLCDFonts) to High(FLCDFonts) do
    FLCDFonts[i].Free;
  for c := Low(FLCDCursors) to High(FLCDCursors) do
    FLCDCursors[c].Free;
  FreeAndNil(FPadding);
  FLCDCursorPos.Free;
  inherited;
end;

// --------
// Methods
// --------

procedure TLCD5x7._ClearLine(y: Byte);
var
  C: Byte;
begin
  for C := 0 to FLCDColumns - 1 do
    FCharBuf[C, Y] := $00;
end;

procedure TLCD5x7._GenerateFont(ab: array of TBitmap; af: array of TFontImage; aIndex: Byte);
var
  C, R, X, Y: Integer;
begin
  with ab[aIndex] do
    begin
      Width  := LCD_CHAR_WIDTH  * (FCellSize + FCellGap) + FCellGap;
      Height := FLcdMatrixHeight * (FCellSize + FCellGap) + FCellGap;
      Canvas.Brush.Style := bsSolid;
      if FCellGap > 0 then
        begin
          Canvas.Brush.Color := FLCDPanelColor;
          Canvas.FillRect(Rect(0, 0, Width, Height));
        end;
      Y := FCellGap;
      for R := 0 to FLcdMatrixHeight - 1 do
        begin
          X := FCellGap;
          for C := 0 to LCD_CHAR_WIDTH - 1 do
            begin
              if (af[aIndex][C] and (1 shl R)) > 0 then
                Canvas.Brush.Color := FLCDForeColor
              else
                Canvas.Brush.Color := FLCDBackColor;
              Canvas.FillRect(Rect(X, Y, X + FCellSize, Y + FCellSize));
              Inc(X, FCellSize + FCellGap);
            end;
          Inc(Y, FCellSize + FCellGap);
        end;
    end;
end;

procedure TLCD5x7.APrint(address: word; const s: TRawString);
var
  i, offset: Word;
  x, y: Byte;
begin
  if Length(s) = 0 then
    Exit;
  for i := 0 to Length(s)-1 do
    begin
      offset := address + i;
      if offset >= (FLCDColumns * FLCDRows) then
        Break;
      x := offset mod FLCDColumns;
      y := offset div FLCDColumns;
      FCharBuf[x, y] := Byte(Ord(s[i + 1]));
    end;
  Invalidate;
end;

procedure TLCD5x7.BlinkTimer(Sender: TObject);
begin
  FBlink := not FBlink;
  if FShowCursor then
    Invalidate;
end;

function TLCD5x7.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
begin
  result := True;
  if Align in [alNone, alLeft, alRight] then
    NewWidth  := FWidth;
  if Align in [alNone, alTop, alBottom] then
    NewHeight := FHeight;
end;

procedure TLCD5x7.ChangeBuf;
var
  i: Integer;
begin
  SetLength(FCharBuf, FLCDColumns);
  for i := Low(FCharBuf) to High(FCharBuf) do
    SetLength(FCharBuf[i], FLCDRows);
end;

procedure TLCD5x7.ChangeLCDColor;
begin
  case FLCDColor of
    // Black on Green without Backlight
    lcdcStandard:
      begin
        FLCDForeColor  := LCD_DEFAULT_FORECOLOR;
        FLCDBackColor  := LCD_DEFAULT_BACKCOLOR;
        FLCDPanelColor := LCD_DEFAULT_PANELCOLOR;
      end;
    // White on Blue
    lcdcWhiteOnBlue:
      begin
        FLCDForeColor  := $00FFFFF0;
        FLCDBackColor  := $00F03030;
        FLCDPanelColor := $00FF8000;
      end;
    // White on Black
    lcdcWhiteOnBlack:
      begin
        FLCDForeColor  := $00FFFFF0;
        FLCDBackColor  := $00603030;
        FLCDPanelColor := $00302020;
      end;
    // Black on Orange
    lcdcBlackOnOrange:
      begin
        FLCDForeColor  := $00303020;
        FLCDBackColor  := $002080E0;
        FLCDPanelColor := $003090FF;
      end;
    // Black on White
    lcdcBlackOnWhite:
      begin
        FLCDForeColor  := $00202020;
        FLCDBackColor  := $00E8E8E8;
        FLCDPanelColor := $00F8F8F8;
      end;
  end;
end;

procedure TLCD5x7.ChangeSize;
begin
  // Width
  FWidth  := (FLCDColumns * (LCD_CHAR_WIDTH  + 1) + 1) * (FCellSize + FCellGap) + FCellGap;
  FWidth  := FWidth + (FPadding.Left * FBevelSize) + (FPadding.Right * FBevelSize);
  // Height
  FHeight := (FLCDRows    * (FLcdMatrixHeight + 1) + 1) * (FCellSize + FCellGap) + FCellGap;
  FHeight := FHeight + (FPadding.Top * FBevelSize) + (FPadding.Bottom * FBevelSize);
  // Set Bounds
  if Self.AutoSize then
  	SetBounds(Left, Top, FWidth, FHeight);
end;

procedure TLCD5x7.ClearLine(y: Byte);
begin
  _ClearLine(y);
  Invalidate;
end;

procedure TLCD5x7.ClearScreen;
var
  R: Byte;
begin
  for R := 0 to FLCDRows - 1 do
    _ClearLine(R);
  Invalidate;
end;

{$IF CompilerVersion >= 21.0}
{$REGION 'procedure TLCD5x7.CMGestureManagerChanged(var Message: TMessage);'}
  procedure TLCD5x7.CMGestureManagerChanged(var Message: TMessage);
  begin
    if not (csDestroying in ComponentState) then
      begin
        if (Touch.GestureManager <> nil) then
          ControlStyle := ControlStyle + [csGestures]
        else
          ControlStyle := ControlStyle - [csGestures];
      end;
  end;
{$ENDREGION}
{$IFEND}

procedure TLCD5x7.DeleteLine(y, n: Byte);
var
  C, R: Byte;
begin
  if  (n = 0) or (n > FLCDRows) or (y > FLCDRows - 1) then
    Exit;
  for R := y to y + n - 1 do
    begin
      if R >= FLCDRows then
        Continue;
      if R + n < FLCDRows then
        begin
          for C := 0 to FLCDColumns - 1 do
            FCharBuf[C, R] := FCharBuf[C, R + n];
        end
      else
        _ClearLine(R);
    end;
  for R := y + 1 to y + n do
    if R < FLCDRows then
      _ClearLine(R);
  Invalidate;
end;

procedure TLCD5x7.DoSizeChange(Sender: TObject);
begin
  ChangeSize;
  Invalidate;
end;

procedure TLCD5x7.GenerateFont;
var
  i: Integer;
  c: TLCDCursorType;
begin
  for i:=Low(FLCDFonts) to High(FLCDFonts) do
    _GenerateFont(FLCDFonts, FFontImages, i);
  for c:=Low(FLCDCursors) to High(FLCDCursors) do
    _GenerateFont(FLCDCursors, FCursorImages ,Ord(c));
end;

procedure TLCD5x7.GenerateFont(aIndex: Byte);
begin
  _GenerateFont(FLCDFonts, FFontImages, aIndex);
end;

procedure TLCD5x7.LPrint(x, y: Byte; const s: TRawString);
var
  address: Word;
begin
  if Length(s) = 0 then
    Exit;
  address := y * FLCDColumns + x;
  APrint(address, Copy(s, 1, FLCDColumns - x));
end;

procedure TLCD5x7.InsertLine(y, n: Byte);
var
  C, R: Byte;
begin
  if  (n = 0) or (n > FLCDRows) or (y > FLCDRows - 1) then
    Exit;
  for R := y + n downto y + 1 do
    begin
      if R >= FLCDRows then
        Continue;
      if R - n > 0 then
        begin
          for C := 0 to FLCDColumns - 1 do
            FCharBuf[C, R] := FCharBuf[C, R - n];
        end
      else
        _ClearLine(R);
    end;
  for R := y to y + n -1 do
    if R < FLCDRows then
      _ClearLine(R);
  Invalidate;
end;

procedure TLCD5x7.Paint;
var
  IncX, IncY, InitX: Integer;
  C, R, X, Y: Integer;
begin
  inherited;

  Self.Canvas.Brush.Color := FLCDPanelColor;
  Self.Canvas.FillRect(Rect(0, 0, Self.Width, Self.Height));

  IncX := FLCDFonts[0].Width  + FCellSize;
  IncY := FLCDFonts[0].Height + FCellSize;

  Y     := (FPadding.Top  + 1) * (FCellSize + FCellGap);
  InitX := (FPadding.Left + 1) * (FCellSize + FCellGap);

  for R := 0 to FLCDRows - 1 do
    begin
      X := InitX;
      for C := 0 to FLCDColumns - 1 do
        begin
          if FShowCursor and (FBlink or (not FLCDCursorBlink)) and
            (C = FLCDCursorPos.X) and (R = FLCDCursorPos.Y) then
            Self.Canvas.Draw(X, Y, FLCDCursors[FLCDCursorType])
          else
            Self.Canvas.Draw(X, Y, FLCDFonts[FCharBuf[C, R]]);
          Inc(X, IncX);
        end;
      Inc(Y, IncY);
    end;

  if Assigned(FOnPaint) then
    FOnPaint(Self);
end;

procedure TLCD5x7.SetCursorImage(aCursor: TLCDCursorType; Image1, Image2,
  Image3, Image4, Image5: Byte; Generate: Boolean);
begin
  FCursorImages[aCursor][0] := Image1;
  FCursorImages[aCursor][1] := Image2;
  FCursorImages[aCursor][2] := Image3;
  FCursorImages[aCursor][3] := Image4;
  FCursorImages[aCursor][4] := Image5;
  if not Generate then
    Exit;
  _GenerateFont(FLCDCursors, FCursorImages, Ord(aCursor));
  Invalidate;
end;

procedure TLCD5x7.SetFontImage(aIndex, Image1, Image2, Image3, Image4,
  Image5: Byte; Generate: Boolean);
begin
  FFontImages[aIndex][0] := Image1;
  FFontImages[aIndex][1] := Image2;
  FFontImages[aIndex][2] := Image3;
  FFontImages[aIndex][3] := Image4;
  FFontImages[aIndex][4] := Image5;
  if not Generate then
    Exit;
  _GenerateFont(FLCDFonts, FFontImages, aIndex);
  Invalidate;
end;

procedure TLCD5x7.ScrollDown(n: Byte);
var
  C, R: Byte;
begin
  if FLCDRows < n then
    Exit;
  for R := n - 1 downto 0 do
    for C := 0 to FLCDColumns - 1 do
      FCharBuf[C, R + n] := FCharBuf[C, R];
  for R := 0 to n - 1 do
    _ClearLine(R);
  Invalidate;
end;

procedure TLCD5x7.ScrollUp(n: Byte);
var
  C, R: Byte;
begin
  if FLCDRows < n then
    Exit;
  for R := 0 to n - 1 do
    for C := 0 to FLCDColumns - 1 do
      FCharBuf[C, R] := FCharBuf[C, R + n];
  for R := 0 to n - 1 do
    _ClearLine(R + n);
  Invalidate;
end;

// ---------------
// Access Methods
// ---------------

function TLCD5x7.GetBitmap: TBitmap;
var
  R: TRect;
begin
  FBitmap.SetSize(Self.Width, Self.Height);
  R := Rect(0, 0, Self.Width, Self.Height);
  FBitmap.Canvas.CopyRect(R, Self.Canvas, R);
  result := FBitmap;
end;

procedure TLCD5x7.SetCDMatrixType(const Value: TLCDMatrixType);
begin
  FLCDMatrixType := Value;
  if FLCDMatrixType = lcdm5x7 then
    FLCDMatrixHeight := 7
  else
    FLCDMatrixHeight := 8;
  SetCursorImages;
  Invalidate;
end;

procedure TLCD5x7.SetCellGap(const Value: Byte);
begin
  if FCellGap = Value then
    Exit;
  FCellGap := Value;
  FBevelSize := FCellSize + (FCellGap * 2);
  ChangeSize;
  GenerateFont;
  Invalidate;
end;

procedure TLCD5x7.SetCellSize(const Value: Byte);
begin
  if FCellSize = Value then
    Exit;
  FCellSize := Value;
  FBevelSize := FCellSize + (FCellGap * 2);
  ChangeSize;
  GenerateFont;
  Invalidate;
end;

procedure TLCD5x7.SetCursorImages;
begin
  if FMatrixType = lcdm5x7 then
    begin
      SetCursorImage(lcduUnderline      , $40, $40, $40, $40, $40, False);
      SetCursorImage(lcduDoubleUnderLine, $60, $60, $60, $60, $60, False);
    end
  else
    begin
      SetCursorImage(lcduUnderline      , $80, $80, $80, $80, $80, False);
      SetCursorImage(lcduDoubleUnderLine, $C0, $C0, $C0, $C0, $C0, False);
    end;
end;

procedure TLCD5x7.SetCursorBlink(const Value: Boolean);
begin
  if FLCDCursorBlink = Value then
    Exit;
  FLCDCursorBlink := Value;
  Invalidate;
end;

procedure TLCD5x7.SetCursorPos(const Value: TLCDCursorPos);
begin
  if FLCDCursorPos = Value then
    Exit;
  FLCDCursorPos := Value;
  Invalidate;
end;

procedure TLCD5x7.SetCursorType(const Value: TLCDCursorType);
begin
  if FLCDCursorType = Value then
    Exit;
  FLCDCursorType := Value;
  Invalidate;
end;

procedure TLCD5x7.SetLCDBackColor(const Value: TColor);
begin
  if FLCDBackColor = Value then
    Exit;
  FLCDBackColor := Value;
  FLCDColor := lcdcCustom;
end;

procedure TLCD5x7.SetLCDColor(const Value: TLCDColor);
begin
  if FLCDColor = Value then
    Exit;
  FLCDColor := Value;
  ChangeLCDColor;
  GenerateFont;
  Invalidate;
end;

procedure TLCD5x7.SetLCDColumns(const Value: Byte);
begin
  if FLCDColumns = Value then
    Exit;
  if Value < 1 then
    FLCDColumns := 1
  else
    FLCDColumns := Value;
  FLCDCursorPos.MaxX := FLCDColumns - 1;
  ChangeBuf;
  ChangeSize;
  Invalidate;
end;

procedure TLCD5x7.SetLCDForeColor(const Value: TColor);
begin
  if FLCDForeColor = Value then
    Exit;
  FLCDForeColor := Value;
  FLCDColor := lcdcCustom;
end;

procedure TLCD5x7.SetLCDPanelColor(const Value: TColor);
begin
  if FLCDPanelColor = Value then
    Exit;
  FLCDPanelColor := Value;
  FLCDColor := lcdcCustom;
end;

procedure TLCD5x7.SetLCDRows(const Value: Byte);
begin
  if FLCDRows = Value then
    Exit;
  if Value < 1 then
    FLCDRows := 1
  else
    FLCDRows := Value;
  FLCDCursorPos.MaxY := FLCDRows - 1;
  ChangeBuf;
  ChangeSize;
  Invalidate;
end;

procedure TLCD5x7.SetPadding(const Value: TPadding);
begin
  if FPadding <> Value then
    FPadding.Assign(Value);
end;

procedure TLCD5x7.SetShowCursor(const Value: Boolean);
begin
  if FShowCursor = Value then
    Exit;
  FShowCursor := Value;
  Invalidate;
end;

end.
